{*******************************************************}
{                                                       }
{         Progress form with two progress bar           }
{                                                       }
{         Copyright (c) FINANCES Ltd                    }
{         2003   ver. 2                                 }
{                                                       }
{         www.finances-ltd.narod.ru                     }
{         finances-ltd@narod.ru                         }
{                                                       }
{*******************************************************}

unit zProgress;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, commctrl, StdCtrls, ExtCtrls,
  Buttons, ComCtrls, MMSystem;

type

  TProgForm = class;

  TBackMode = (bmDefault, bmWallPaper, bmGradientVert, bmGradientHorz, bmGradientDblVert, bmGradientDblHorz);

  TOnCancelEvent = procedure(Sender: TObject; var Cancel: Boolean) of object;

  TzProgress = class(TComponent)
  private
    FOnExecute: TNotifyEvent;
    FOnCanceled: TNotifyEvent;
    FAfterClose: TNotifyEvent;
    FOnShow: TNotifyEvent;
    FExecuted: Boolean;
    FProgressBarCount: Integer;
    FTopMessage, FPrompt: string;
    FTopPosition, FPosition: Integer;
    FTopMax, FMax: Integer;
    FStep: Integer;
    FFinishSound: Boolean;
    FAutoClose: Boolean;
    FFont: TFont;
    FEnableCancel: Boolean;
    FProgForm: TProgForm;
    FPicture: TPicture;
    FOnCancel: TOnCancelEvent;
    FBackMode: TBackMode;
    FWallPaper: TBitmap;
    FColor: TColor;
    FStartColor,
    FEndColor: TColor;

    procedure SetProgressBarCount(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetTopMax(Value: Integer);
    procedure SetTopPosition(Value: Integer);
    procedure SetPrompt(Value: string);
    procedure SetTopMessage(Value: string);
    procedure SetPicture(Value: TPicture);
    procedure PictureChanged(Sender: TObject);
    function GetLeft: Integer;
    function GetTop: Integer;
    function GetWidth: Integer;
    function GetHeight: Integer;
    procedure SetWallPaper(Value: TBitmap);
    function GetWallPaper: TBitmap;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Canceled: Boolean;
    constructor   Create(AOwner : TComponent); override;
    destructor    Destroy; override;
    function Execute: Boolean;
    procedure Show;
    function Showing: Boolean;
    procedure Hide;
    procedure Close;

    property Busy: Boolean read FExecuted;
    property Left: Integer read GetLeft;
    property Top: Integer read GetTop;
    property Width: Integer read GetWidth;
    property Height: Integer read GetHeight;
    procedure DrawTime;
  published
    { Published declarations }
    property ProgressBarCount: Integer read FProgressBarCount write SetProgressBarCount default 1;
    property Prompt: string read FPrompt write SetPrompt;
    property Title: string read FTopMessage write SetTopMessage;
    property AutoClose: Boolean read FAutoClose write FAutoClose default True;
    property Font: TFont read FFont write FFont;
    property EnableCancel: Boolean read FEnableCancel write FEnableCancel default True;
    property TopPosition: Integer read FTopPosition write SetTopPosition default 0;
    property Position: Integer read FPosition write SetPosition default 0;
    property TopMax: Integer read FTopMax write SetTopMax default 100;
    property Max: Integer read FMax write SetMax default 100;
    property Step: Integer read FStep write FStep default 10;
    property FinishSound: Boolean read FFinishSound write FFinishSound default True;
    property Picture: TPicture read FPicture write SetPicture;
    property BackMode: TBackMode read FBackMode write FBackMode default bmDefault;
    property StartColor: TColor read FStartColor write FStartColor Default clSilver;
    property EndColor: TColor read FEndColor write FEndColor Default clWhite;
    property WallPaper: TBitmap read GetWallPaper write SetWallPaper;
    property Color: TColor read FColor write FColor Default clInfoBk;

    property AfterClose: TNotifyEvent read FAfterClose write FAfterClose;
    property OnCancel: TOnCancelEvent read FOnCancel write FOnCancel;
    property OnCanceled: TNotifyEvent read FOnCanceled write FOnCanceled;
    property OnExecute: TNotifyEvent read FOnExecute write FOnExecute;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  TProgForm = class(TCustomForm)
  private
    FAfterClose: TNotifyEvent;
    lblTitle: TLabel;
    lblText: TLabel;
    iconBitmap: TImage;
    FButton: TSpeedButton;
    Prog1, Prog3: TProgressBar;
    TimeBox: TPaintBox;
    StartTime: TTime;
    CloseTimer: TTimer;

    procedure FormPaint(Sender: TObject);
    procedure FormButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    Procedure OnChange(Sender: TObject);
//    procedure WMNCHitTest(var M: TWMNCHitTest); message wm_NCHitTest;
  protected
    Procedure CreateParams(Var Params: TCreateParams); override;
  public
    FExecuteProc: TNotifyEvent;
    MyComp: TzProgress;
    constructor CreateNew(AOwner: TComponent; Dummy: Integer = 0); override;
    destructor  Destroy; override;
    procedure ShowForm(Modal : Boolean);
    procedure CloseForm(Sender: TObject);

    property AfterClose: TNotifyEvent read FAfterClose write FAfterClose;
  End;

const Flags = SND_RESOURCE or SND_SYNC;

procedure Register;

implementation
{$R zProgress.RES}

constructor TzProgress.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FExecuted := False;
  FProgressBarCount := 1;
  FPrompt := '';
  FTopMessage := '';
  FTopPosition := 0;
  FPosition := 0;
  FTopMax := 100;
  FMax := 100;
  FStep := 10;
  FFinishSound := True;
  FAutoClose := True;
  FFont := TFont.Create;
  FEnableCancel := True;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FStartColor := clSilver;
  FEndColor := clWhite;
  FBackMode := bmDefault;
  FStartColor := clSilver;
  FEndColor := clWhite;
  FColor := clInfoBk;
end;

destructor TzProgress.Destroy;
begin
  FPicture.Free;
  FFont.Free;
  inherited Destroy;
end;

function TzProgress.Execute: Boolean;
begin
  Result := False;
  if FExecuted or not Assigned(FOnExecute) then Exit;
  FPosition := 0;
  FTopPosition := 0;
  with TProgForm.CreateNew(nil) do begin
    //AlphaBlendValue := 100;
    FExecuteProc := FOnExecute;
    MyComp := Self;
    Canceled := False;
    FAfterClose := Self.FAfterClose;
    ShowForm(true);
    Result := not Canceled;
  end;
end;

procedure TzProgress.Show;
begin
  if {FExecuted and} Assigned(FProgForm) then begin
    if Showing then Exit;
    FProgForm.Show;
    FProgForm.Update;
  end
  else with TProgForm.CreateNew(nil) do begin
    Self.FPosition := 0;
    Self.FTopPosition := 0;
    MyComp := Self;
    Canceled := False;
    FAfterClose := Self.FAfterClose;
    ShowForm(False);
  end;
end;

procedure TzProgress.Hide;
begin
  if Assigned(FProgForm) then
    FProgForm.Hide;
end;

function TzProgress.Showing: Boolean;
begin
  Result := Assigned(FProgForm) and FProgForm.Visible;
end;

procedure TzProgress.Close;
begin
  if FExecuted and Assigned(FProgForm) and not FProgForm.CloseTimer.Enabled and
     not (fsModal in FProgForm.FormState) then begin
    FProgForm.CloseForm(FProgForm);
//    FProgForm.CloseTimer.Interval := 1;
//    FProgForm.CloseTimer.Enabled := True;
//    Application.ProcessMessages;
  end;
end;

procedure TzProgress.SetProgressBarCount(Value: Integer);
begin
  if Value <> FProgressBarCount then begin
    if Value in [1..2] then
      FProgressBarCount := Value
    else begin
      if Value < 1 then
        FProgressBarCount := 1
      else FProgressBarCount := 2;
    end;
  end;
end;

procedure TzProgress.SetMax(Value: Integer);
begin
  if csDestroying in ComponentState then Exit;
  if Value <> FMax then begin
    FMax := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then
      FProgForm.Prog3.Max := FMax;
  end;
end;

procedure TzProgress.SetPosition(Value: Integer);
begin
  if csDestroying in ComponentState then Exit;
  if Value <> FPosition then begin
    FPosition := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then begin
      FProgForm.Prog3.Position := FPosition;

      if FAutoClose and (FProgressBarCount = 1) and
         (FProgForm.Prog3.Position = FProgForm.Prog3.Max) then
         FProgForm.CloseTimer.Enabled := True
      else DrawTime;
    end;
  end;
end;

procedure TzProgress.SetTopMax(Value: Integer);
begin
  if Value <> FTopMax then begin
    FTopMax := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then
      FProgForm.Prog1.Max := FTopMax;
  end;
end;

procedure TzProgress.SetTopPosition(Value: Integer);
begin
  if Value <> FTopPosition then begin
    FTopPosition := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then with FProgForm do begin
      if Prog1.Visible then begin
        Prog1.Position := FTopPosition;

        if FAutoClose and (FProgressBarCount = 2) and
           (Prog1.Position = Prog1.Max) then //FProgForm.Close
          FProgForm.CloseTimer.Enabled := True
        else DrawTime;
      end;
    end;
  end;
end;

procedure TzProgress.SetPrompt(Value: string);
begin
  if csDestroying in ComponentState then Exit;
  if Value <> FPrompt then begin
    FPrompt := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then with FProgForm, lblText do begin
      Caption := Value;
      Update;
    end;
  end;
end;

procedure TzProgress.SetTopMessage(Value: string);
begin
  if Value <> FTopMessage then begin
    FTopMessage := Value;
    if FExecuted and not Canceled and Assigned(FProgForm) then with FProgForm.lblTitle do begin
      Caption := Value;
      Update;
    end;
  end;
end;

procedure TzProgress.DrawTime;
var s: string;
begin
  if (FBackMode = bmDefault) and FExecuted and not Canceled and Assigned(FProgForm) then with FProgForm do begin
    s := TimeToStr(Time - StartTime);
    s := Copy(s, Pos(':', s) + 1, MAXINT);
    if Pos(' ', s) > 0 then s := Copy(s, 1, Pos(' ', s) - 1);
    with TimeBox.Canvas do begin
      TextRect(ClipRect, 4, 2, s);
    end;
    Application.ProcessMessages;
  end;
end;

procedure TzProgress.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TzProgress.PictureChanged(Sender: TObject);
begin
  if FExecuted and not Canceled and Assigned(FProgForm) then begin
    FProgForm.iconBitmap.Picture.Assign(FPicture);
  end;
end;

function TzProgress.GetLeft: Integer;
begin
  if Assigned(FProgForm) then
    Result := FProgForm.Left
  else Result := 0;
end;

function TzProgress.GetTop: Integer;
begin
  if Assigned(FProgForm) then
    Result := FProgForm.Top
  else Result := 0;
end;

function TzProgress.GetWidth: Integer;
begin
  if Assigned(FProgForm) then
    Result := FProgForm.Width
  else Result := 0;
end;

function TzProgress.GetHeight: Integer;
begin
  if Assigned(FProgForm) then
    Result := FProgForm.Height
  else Result := 0;
end;

procedure TzProgress.SetWallPaper(Value: TBitmap);
begin
  if FWallPaper = nil then
    FWallPaper := TBitmap.Create;
  FWallPaper.Assign(Value);
end;

function TzProgress.GetWallPaper: TBitmap;
begin
  if FWallPaper = nil then
    FWallPaper := TBitmap.Create;
  Result := FWallPaper;
end;

/////////////////////////////////////  TPROGFORM //////////////////////////////////////

constructor TProgForm.CreateNew(AOwner: TComponent; Dummy: Integer = 0);
Begin
  inherited;

  OnKeyDown    := FormKeyDown;
  OnClose      := FormClose;
  OnActivate   := FormActivate;
  OnDeactivate := OnChange;
  OnShow       := OnChange;

  BorderStyle  := bsNone;
  FormStyle    := fsStayOnTop;
  OnPaint      := FormPaint;
  Color        := clInfoBk;
  Font.Name    := 'Tahoma';

  lblTitle   := TLabel.Create(Self);
  lblText    := TLabel.Create(Self);
  iconBitmap := TImage.Create(Self);
  FButton    := TSpeedButton.Create(Self);
  Prog1      := TProgressBar.Create(Self);
  Prog3      := TProgressBar.Create(Self);
  TimeBox    := TPaintBox.Create(Self);
  CloseTimer := TTimer.Create(Self);

  CloseTimer.Enabled := False;
  CloseTimer.Interval := 100;
  CloseTimer.OnTimer := CloseForm;

  lblTitle.Parent      := Self;
  lblTitle.ParentColor := True;
  lblTitle.ParentFont  := True;
  lblTitle.AutoSize    := True;
  lblTitle.Left        := 52;
  lblTitle.Top         := 8;

  lblText.Parent      := Self;
  lblText.ParentColor := True;
  lblText.ParentFont  := True;
  lblText.AutoSize    := True;

  iconBitmap.Parent      := Self;
  iconBitmap.Transparent := True;
  iconBitmap.Left        := 10;
  iconBitmap.Top         := 8;

  with FButton do begin
    Parent := Self;
//    Flat := True;
    Top := 3;
    Height := 18;
    Width := 18;
    Caption := 'X';
    Left := Self.Width - 5 - Width;
    Anchors := [akTop] + [akRight];
    Font.Style := [fsBold];
    Font.Name := 'Tahoma';
    Font.Color := clSilver;
    OnClick := FormButtonClick;
  end;

  with Prog1 do begin
    Parent := Self;
    Width := 180;
    Left := 52;
    Height := 10;
  end;

  with Prog3 do begin
    Parent := Self;
    Width := 180;
    Left := 52;
    Height := 10;
  end;

  with TimeBox do begin
    Parent := Self;
    Color := clInfoBk;
    Font.Color := clMaroon;
    Font.Name := 'Tahoma';
    Font.Style := [fsBold];
    Left := 7;
    Top := 52;
    Width := 41;
    Height := 14;
  end;
  StartTime := Time;

  Application.ProcessMessages;
End;

procedure TProgForm.FormButtonClick(Sender: TObject);
var Cancel: Boolean;
begin
  if Assigned(MyComp.FOnCancel) then begin
    Cancel := True;
    MyComp.FOnCancel(MyComp, Cancel);
    if not Cancel then Exit;
  end;
  MyComp.Canceled := True;

  //lblTitle.Hide;
  lblText.Hide;
  //iconBitmap.Hide;
  //FButton.Hide;
  Prog1.Hide;
  Prog3.Hide;
  TimeBox.Hide;

  with lblTitle do begin
    Alignment := taCenter;
    WordWrap := True;
    Font.Name := 'Tahoma';
    Font.Size := 14;
    Font.Color := clMaroon;
    Font.Style := [fsBold];
    Top := Top - 5;
    Left := Left + 10;
    Width := Self.Width - Left - 20;
    Caption := 'Process canceled by user';
    Top := Self.Height div 2 - Height div 2;
  end;
  CloseTimer.Interval := 1000;
  CloseTimer.Enabled := True;
end;


procedure TProgForm.FormPaint(Sender: TObject);
const mxcolors=100;
type TGrArray=array[1..mxcolors] of tcolor;

var TempRegion: HRGN;
    ga:tgrarray;
    x, y, v, h: Integer;

  procedure HorzOneWay;
  var
    Banda : TRect;
    I         : Integer;
  begin
     Banda.Left := 0;
     Banda.Right := Width;
     with canvas do
     for I := 0 to mxcolors-1 do
     begin
         Banda.Top    := MulDiv (I    , Height, mxcolors);
         Banda.Bottom := MulDiv (I + 1, Height, mxcolors);
         Brush.Color := ga[i+1];
         FillRect (Banda);
     end;
  end;

  procedure VertOneWay;
  var
    Banda : TRect;
    I         : Integer;
  begin
     Banda.Top := 0;
     Banda.Bottom := Height;
     with canvas do
     for I := 0 to mxcolors-1 do
     begin
         Banda.Left    := MulDiv (I    , Width, mxcolors);
         Banda.Right := MulDiv (I + 1, Width, mxcolors);
         Brush.Color := ga[i+1];
         FillRect (Banda);
     end;
  end;

  procedure HorzTwoWay;
  var
    Banda : TRect;
    q,I         : Integer;
  begin
     Banda.Left := 0;
     Banda.Right := Width;
     q:= mxcolors div 2;
     with canvas do
     for I := 0 to q-1 do
     begin
         Banda.Top    := MulDiv (I    , Height, mxcolors);
         Banda.Bottom := MulDiv (I + 1, Height, mxcolors);
         Brush.Color := ga[2*(i+1)];
         FillRect (Banda);
     end;
     with canvas do for I := q to mxcolors-1 do
     begin
         Banda.Top    := MulDiv (I    , Height, mxcolors);
         Banda.Bottom := MulDiv (I + 1, Height, mxcolors);
         Brush.Color := ga[mxcolors+1 - 2*(i-q+1)];
         FillRect (Banda);
     end;
  end;

  procedure VertTwoWay;
  var
    Banda : TRect;
    I,q         : Integer;
  begin
     Banda.Top := 0;
     Banda.Bottom := Height;
     q:=mxcolors div 2;
     with canvas do
     for I := 0 to q -1 do
     begin
         Banda.Left    := MulDiv (I    , Width, mxcolors);
         Banda.Right := MulDiv (I + 1, Width, mxcolors);
         Brush.Color := ga[2*(i+1)];
         FillRect (Banda);
     end;
     with canvas do
     for I := q  to mxcolors-1 do
     begin
         Banda.Left    := MulDiv (I    , Width, mxcolors);
         Banda.Right := MulDiv (I + 1, Width, mxcolors);
         Brush.Color := ga[mxcolors+1 - 2*(i-q+1)];
         FillRect (Banda);
     end;
  end;

  procedure CalcColGradients(col1,col2:tcolor;n:integer);
  var i:integer;
       ar,ag,ab,br,bb,bg:integer;
       kr,kg,kb:real;
  begin
     ar:=GetRValue(col1);br:=GetRValue(col2);
     AG:=GetGValue(col1);bG:=GetGValue(col2);
     AB:=GetBValue(col1);bB:=GetBValue(col2);
     kr:=(br-ar) /n;
     kg:=(bg-ag) /n;
     kb:=(bb-ab) /n;
     for i:=0 to n-1 do
     ga[i+1]:=rgb(trunc(ar + i*kr),trunc(ag+i*kg),trunc(ab+i*kb));
  end;

  procedure FillOneWay(Vert: Boolean);
  begin
    if Vert then begin
      CalcColGradients(MyComp.FStartColor, MyComp.FEndColor, mxcolors);
      VertOneWay;
    end
    else begin
      CalcColGradients(MyComp.FStartColor, MyComp.FEndColor, mxcolors);
      HorzOneWay;
    end;
  end;

  procedure FillTwoWay(Vert: Boolean);
  begin
    if Vert then begin
      CalcColGradients(MyComp.FStartColor, MyComp.FEndColor, mxcolors);
      VertTwoWay;
    end
    else begin
      CalcColGradients(MyComp.FStartColor, MyComp.FEndColor, mxcolors);
      HorzTwoWay;
    end;
  end;

Begin
  case MyComp.FBackMode of
    bmWallPaper: if Assigned(MyComp.FWallPaper) then begin
                    v := Height div MyComp.FWallPaper.Height + 1;
                    h := Width div MyComp.FWallPaper.Width + 1;
                    for x := 0 to h do
                      for y := 0 to v do
                        Canvas.Draw(x * MyComp.FWallPaper.Width,
                                    y * MyComp.FWallPaper.Height,
                                    MyComp.FWallPaper);
                 end;
    bmGradientVert: FillOneWay(True);
    bmGradientHorz: FillOneWay(False);
    bmGradientDblVert: FillTwoWay(True);
    bmGradientDblHorz: FillTwoWay(False);
  end;

  with Canvas.Brush do Begin
    Color := clBlack;
    Style := bsSolid;
  end;
  TempRegion := CreateRectRgn(0,0,1,1);
  GetWindowRgn(Handle, TempRegion);
  FrameRgn(Canvas.Handle, TempRegion, Canvas.Brush.handle, 1, 1);
  DeleteObject(TempRegion);
  MyComp.DrawTime;
End;


procedure TProgForm.ShowForm(Modal : boolean);
var FormRegion: HRGN;
    i: Integer;
begin
  MyComp.FExecuted := True;
  MyComp.FProgForm := Self;
  FButton.Visible := MyComp.FEnableCancel;
  with MyComp do begin
    Prog1.Max := FTopMax;
    Prog3.Max := FMax;
    Prog1.Step := FStep;
    Prog3.Step := FStep;
    Prog1.Position := 0;
    Prog3.Position := 0;
    lblTitle.Font.Assign(Font);
    lblTitle.Font.Style  := [fsBold];
    lblText.Font.Assign(Font);
  end;

//  position := poOwnerFormCenter;

//  FormRegion := 0;
  lblTitle.Caption := MyComp.Title;
  if lblTitle.Caption = '' then
    lblTitle.Caption := Application.Title;
  lblText.Caption := MyComp.Prompt;
  lblText.Left        := lblTitle.Left;
  lblText.Top         := lblTitle.Top + lblTitle.Height + 3;
  if lblTitle.Width > lblText.Width then
    i := lblTitle.Width
  else i := lblText.Width;

  if MyComp.FPicture.Graphic = nil then
    iconBitmap.Picture.Icon.Assign(Application.Icon)
  else iconBitmap.Picture.Assign(MyComp.FPicture);

  Width := lblTitle.Left + i + 10 + FButton.Width + 10;
  if Width < Prog1.Left + Prog1.Width + 20 then
    Width := Prog1.Left + Prog1.Width + 20;
  ClientHeight := lblText.Top + lblText.Height + 10 * 4;

  if MyComp.ProgressBarCount > 1 then begin
    Prog1.Top := ClientHeight - 30;
    Prog3.Top := ClientHeight - 20;
  end
  else begin
    Prog1.Visible := False;
    Prog3.Top := ClientHeight - 25;
  end;
  TimeBox.Top := Prog3.Top + Prog3.Height - TimeBox.Height;

  //with TCustomForm(MyComp.Owner) do
  if Assigned(Screen.ActiveCustomForm) then
    with Screen.ActiveCustomForm do
      Self.SetBounds(Left + Width div 2 - Self.Width div 2,
                Top + Height div 2 - Self.Height div 2,
                Self.Width, Self.Height)
  else with Screen do
      Self.SetBounds(Width div 2 - Self.Width div 2,
                Height div 2 - Self.Height div 2,
                Self.Width, Self.Height);
  if Assigned(MyComp.OnShow) then MyComp.OnShow(Self);

  FormRegion := CreateRoundRectRgn(0, 0, Self.Width, Self.Height, 7, 7);
  SetWindowRgn(Handle, FormRegion, True);

  if Modal then ShowModal
  else begin
    TCustomForm(MyComp.Owner).Enabled := False;
    Show;
  end;
end;

procedure TProgForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Shift = []) and (Key = VK_ESCAPE) and (FButton.Visible) and
     (Screen.ActiveCustomForm = Self) then
    FButton.Click;
end;

procedure TProgForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  {MyComp.FExecuted := False;
  if MyComp.Canceled then
    ModalResult := mrCancel
  else ModalResult := mrOK;
  TCustomForm(MyComp.Owner).Enabled := True;
  MyComp.FProgForm := nil;}
  Action := CaFree; // Destroy will be called automatically ....
end;

destructor TProgForm.Destroy;
begin
  MyComp.FExecuted := False;
  TCustomForm(MyComp.Owner).Enabled := True;
  if MyComp.Canceled then
    ModalResult := mrCancel
  else ModalResult := mrOK;
  MyComp.FProgForm := nil;
{  OnActivate   := nil;
  OnDeactivate := nil;
  OnShow       := nil;
  OnKeyDown    := nil;
  OnClose      := nil;

  MyComp.FProgForm := nil;

  lblTitle.Free;
  lblText.Free;
  iconBitmap.Free;
  FButton.Free;
  Prog1.Free;
  Prog3.Free;
  TimeBox.Free;
  CloseTimer.Free;}
//  Application.ProcessMessages;
  inherited;
end;

{procedure TProgForm.WMNCHitTest(var M: TWMNCHitTest);
begin
   inherited; //SetCursorPos
   if (not PtInRect(FButton.BoundsRect, ScreenToClient(Mouse.CursorPos))) and
      (M.Result = htClient) then begin
     M.Result := htCaption;
     Application.ProcessMessages;
   end;
end;}

procedure TProgForm.FormActivate(Sender: TObject);
begin
  MyComp.DrawTime;
  TimeBox.Refresh;
  if (fsModal in FormState) and Assigned(MyComp.OnExecute) then begin
    MyComp.OnExecute(MyComp);
    CloseTimer.Interval := 1;
    CloseTimer.Enabled := True;
  end;
end;

procedure TProgForm.CloseForm(Sender: TObject);
begin
  CloseTimer.Enabled := False;
  MyComp.FExecuted := False;
  MyComp.FProgForm := nil;
  TCustomForm(MyComp.Owner).Enabled := True;
  Hide;
  Application.ProcessMessages;
  if not (csDestroying in MyComp.ComponentState) then begin
    if MyComp.FFinishSound then PlaySound('SOUND_0', 0, Flags);
  end;
  {if MyComp.Canceled then
    ModalResult := mrCancel
  else ModalResult := mrOK;}
  Close;
//  Application.ProcessMessages;
  if Assigned(FAfterClose) then
    FAfterClose(Self);
//  MyComp.FProgForm := nil;
//  if MyComp.FFinishSound then PlaySound('SOUND_0', 0, Flags);
end;

Procedure TProgForm.OnChange(Sender: TObject);
Begin
  if Assigned(MyComp) then Color := MyComp.FColor;
  SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
End;

Procedure TProgForm.CreateParams(Var Params: TCreateParams);
const
  CS_DROPSHADOW = $00020000;
Begin
  Inherited CreateParams(Params);

  Params.Style     := (Params.Style and not WS_CAPTION) or WS_POPUP;
  Params.ExStyle   := Params.ExStyle or WS_EX_TOOLWINDOW or WS_EX_NOACTIVATE or WS_EX_TOPMOST;
  if (Win32MajorVersion = 5) and (Win32MinorVersion = 1) then
    Params.WindowClass.Style := Params.WindowClass.Style or CS_DROPSHADOW;
End;

/////////////////////////////////////  END TPROGFORM //////////////////////////////////

procedure Register;
begin
  RegisterComponents('zPage', [TzProgress]);
end;


end.
